@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2018 Hans Petter Jansson
+/* Copyright (C) 2018-2021 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that turns images into character art.
  *
@@ -20,8 +20,8 @@
 #include "config.h"
 
 #include <string.h>  /* memset, memcpy */
-#include "chafa/chafa.h"
-#include "chafa/chafa-private.h"
+#include "chafa.h"
+#include "internal/chafa-private.h"
 
 /**
  * SECTION:chafa-canvas-config
@@ -50,6 +50,13 @@
  * @CHAFA_CANVAS_MODE_MAX: Last supported canvas mode plus one.
  **/
 
+/**
+ * ChafaColorExtractor:
+ * @CHAFA_COLOR_EXTRACTOR_AVERAGE: Use the average colors of each symbol's coverage area.
+ * @CHAFA_COLOR_EXTRACTOR_MEDIAN: Use the median colors of each symbol's coverage area.
+ * @CHAFA_COLOR_EXTRACTOR_MAX: Last supported color extractor plus one.
+ **/
+
 /**
  * ChafaColorSpace:
  * @CHAFA_COLOR_SPACE_RGB: RGB color space. Fast but imprecise.
@@ -65,6 +72,22 @@
  * @CHAFA_DITHER_MODE_MAX: Last supported dither mode plus one.
  **/
 
+/**
+ * ChafaPixelMode:
+ * @CHAFA_PIXEL_MODE_SYMBOLS: Pixel data is approximated using character symbols ("ANSI art").
+ * @CHAFA_PIXEL_MODE_SIXELS: Pixel data is encoded as sixels.
+ * @CHAFA_PIXEL_MODE_MAX: Last supported pixel mode plus one.
+ **/
+
+/**
+ * ChafaOptimizations:
+ * @CHAFA_OPTIMIZATION_REUSE_ATTRIBUTES: Suppress redundant SGR control sequences.
+ * @CHAFA_OPTIMIZATION_SKIP_CELLS: Reserved for future use.
+ * @CHAFA_OPTIMIZATION_REPEAT_CELLS: Use REP sequence to compress repeated runs of similar cells.
+ * @CHAFA_OPTIMIZATION_NONE: All optimizations disabled.
+ * @CHAFA_OPTIMIZATION_ALL: All optimizations enabled.
+ **/
+
 /* Private */
 
 void
@@ -76,9 +99,13 @@ chafa_canvas_config_init (ChafaCanvasConfig *canvas_config)
     canvas_config->refs = 1;
 
     canvas_config->canvas_mode = CHAFA_CANVAS_MODE_TRUECOLOR;
+    canvas_config->color_extractor = CHAFA_COLOR_EXTRACTOR_AVERAGE;
     canvas_config->color_space = CHAFA_COLOR_SPACE_RGB;
+    canvas_config->pixel_mode = CHAFA_PIXEL_MODE_SYMBOLS;
     canvas_config->width = 80;
     canvas_config->height = 24;
+    canvas_config->cell_width = 8;
+    canvas_config->cell_height = 8;
     canvas_config->dither_mode = CHAFA_DITHER_MODE_NONE;
     canvas_config->dither_grain_width = 4;
     canvas_config->dither_grain_height = 4;
@@ -88,6 +115,7 @@ chafa_canvas_config_init (ChafaCanvasConfig *canvas_config)
     canvas_config->alpha_threshold = 127;
     canvas_config->work_factor = 0.5;
     canvas_config->preprocessing_enabled = TRUE;
+    canvas_config->optimizations = CHAFA_OPTIMIZATION_ALL;
 
     chafa_symbol_map_init (&canvas_config->symbol_map);
     chafa_symbol_map_add_by_tags (&canvas_config->symbol_map, CHAFA_SYMBOL_TAG_ALL);
@@ -235,11 +263,57 @@ chafa_canvas_config_set_geometry (ChafaCanvasConfig *config, gint width, gint he
     config->height = height;
 }
 
+/**
+ * chafa_canvas_config_get_cell_geometry:
+ * @config: A #ChafaCanvasConfig
+ * @cell_width_out: Location to store cell width in, or %NULL
+ * @cell_height_out: Location to store cell height in, or %NULL
+ *
+ * Returns @config's cell width and height in pixels in the
+ * provided output locations.
+ *
+ * Since: 1.4
+ **/
+void
+chafa_canvas_config_get_cell_geometry (const ChafaCanvasConfig *config, gint *cell_width_out, gint *cell_height_out)
+{
+    g_return_if_fail (config != NULL);
+    g_return_if_fail (config->refs > 0);
+
+    if (cell_width_out)
+        *cell_width_out = config->cell_width;
+    if (cell_height_out)
+        *cell_height_out = config->cell_height;
+}
+
+/**
+ * chafa_canvas_config_set_cell_geometry:
+ * @config: A #ChafaCanvasConfig
+ * @cell_width: Cell width in pixels
+ * @cell_height: Cell height in pixels
+ *
+ * Sets @config's cell width and height in pixels to @cell_width x @cell_height.
+ *
+ * Since: 1.4
+ **/
+void
+chafa_canvas_config_set_cell_geometry (ChafaCanvasConfig *config, gint cell_width, gint cell_height)
+{
+    g_return_if_fail (config != NULL);
+    g_return_if_fail (config->refs > 0);
+    g_return_if_fail (cell_width > 0);
+    g_return_if_fail (cell_height > 0);
+
+    config->cell_width = cell_width;
+    config->cell_height = cell_height;
+}
+
 /**
  * chafa_canvas_config_get_canvas_mode:
  * @config: A #ChafaCanvasConfig
  *
- * Returns @config's #ChafaCanvasMode.
+ * Returns @config's #ChafaCanvasMode. This determines how colors (and
+ * color control codes) are used in the output.
  *
  * Returns: The #ChafaCanvasMode.
  **/
@@ -257,7 +331,8 @@ chafa_canvas_config_get_canvas_mode (const ChafaCanvasConfig *config)
  * @config: A #ChafaCanvasConfig
  * @mode: A #ChafaCanvasMode
  *
- * Sets @config's stored #ChafaCanvasMode to @mode.
+ * Sets @config's stored #ChafaCanvasMode to @mode. This determines how
+ * colors (and color control codes) are used in the output.
  **/
 void
 chafa_canvas_config_set_canvas_mode (ChafaCanvasConfig *config, ChafaCanvasMode mode)
@@ -269,6 +344,46 @@ chafa_canvas_config_set_canvas_mode (ChafaCanvasConfig *config, ChafaCanvasMode
     config->canvas_mode = mode;
 }
 
+/**
+ * chafa_canvas_config_get_color_extractor:
+ * @config: A #ChafaCanvasConfig
+ *
+ * Returns @config's #ChafaColorExtractor. This determines how colors are
+ * approximated in character symbol output.
+ *
+ * Returns: The #ChafaColorExtractor.
+ *
+ * Since: 1.4
+ **/
+ChafaColorExtractor
+chafa_canvas_config_get_color_extractor (const ChafaCanvasConfig *config)
+{
+    g_return_val_if_fail (config != NULL, CHAFA_COLOR_EXTRACTOR_AVERAGE);
+    g_return_val_if_fail (config->refs > 0, CHAFA_COLOR_EXTRACTOR_AVERAGE);
+
+    return config->color_extractor;
+}
+
+/**
+ * chafa_canvas_config_set_color_extractor:
+ * @config: A #ChafaCanvasConfig
+ * @color_extractor: A #ChafaColorExtractor
+ *
+ * Sets @config's stored #ChafaColorExtractor to @color_extractor. This
+ * determines how colors are approximated in character symbol output.
+ *
+ * Since: 1.4
+ **/
+void
+chafa_canvas_config_set_color_extractor (ChafaCanvasConfig *config, ChafaColorExtractor color_extractor)
+{
+    g_return_if_fail (config != NULL);
+    g_return_if_fail (config->refs > 0);
+    g_return_if_fail (color_extractor < CHAFA_COLOR_EXTRACTOR_MAX);
+
+    config->color_extractor = color_extractor;
+}
+
 /**
  * chafa_canvas_config_get_color_space:
  * @config: A #ChafaCanvasConfig
@@ -510,7 +625,6 @@ chafa_canvas_config_get_work_factor (const ChafaCanvasConfig *config)
  *
  * Sets the work/quality tradeoff factor. A higher value means more time
  * and memory will be spent towards a higher quality output.
- *
  **/
 void
 chafa_canvas_config_set_work_factor (ChafaCanvasConfig *config, gfloat work_factor)
@@ -687,3 +801,84 @@ chafa_canvas_config_set_dither_intensity (ChafaCanvasConfig *config, gfloat inte
 
     config->dither_intensity = intensity;
 }
+
+/**
+ * chafa_canvas_config_get_pixel_mode:
+ * @config: A #ChafaCanvasConfig
+ *
+ * Returns @config's #ChafaPixelMode.
+ *
+ * Returns: The #ChafaPixelMode. This determines how pixel graphics are
+ * rendered in the output.
+ *
+ * Since: 1.4
+ **/
+ChafaPixelMode
+chafa_canvas_config_get_pixel_mode (const ChafaCanvasConfig *config)
+{
+    g_return_val_if_fail (config != NULL, CHAFA_PIXEL_MODE_SYMBOLS);
+    g_return_val_if_fail (config->refs > 0, CHAFA_PIXEL_MODE_SYMBOLS);
+
+    return config->pixel_mode;
+}
+
+/**
+ * chafa_canvas_config_set_pixel_mode:
+ * @config: A #ChafaCanvasConfig
+ * @pixel_mode: A #ChafaPixelMode
+ *
+ * Sets @config's stored #ChafaPixelMode to @pixel_mode. This determines
+ * how pixel graphics are rendered in the output.
+ *
+ * Since: 1.4
+ **/
+void
+chafa_canvas_config_set_pixel_mode (ChafaCanvasConfig *config, ChafaPixelMode pixel_mode)
+{
+    g_return_if_fail (config != NULL);
+    g_return_if_fail (config->refs > 0);
+    g_return_if_fail (pixel_mode < CHAFA_PIXEL_MODE_MAX);
+
+    config->pixel_mode = pixel_mode;
+}
+
+/**
+ * chafa_canvas_config_get_optimizations:
+ * @config: A #ChafaCanvasConfig
+ *
+ * Returns @config's optimization flags. When enabled, these may produce
+ * more compact output at the cost of reduced compatibility and increased CPU
+ * use. Output quality is unaffected.
+ *
+ * Returns: The #ChafaOptimizations flags.
+ *
+ * Since: 1.6
+ **/
+ChafaOptimizations
+chafa_canvas_config_get_optimizations (const ChafaCanvasConfig *config)
+{
+    g_return_val_if_fail (config != NULL, CHAFA_OPTIMIZATION_NONE);
+    g_return_val_if_fail (config->refs > 0, CHAFA_OPTIMIZATION_NONE);
+
+    return config->optimizations;
+}
+
+/**
+ * chafa_canvas_config_set_optimizations:
+ * @config: A #ChafaCanvasConfig
+ * @optimizations: A combination of #ChafaOptimizations flags
+ *
+ * Sets @config's stored optimization flags. When enabled, these may produce
+ * more compact output at the cost of reduced compatibility and increased CPU
+ * use. Output quality is unaffected.
+ *
+ * Since: 1.6
+ **/
+void
+chafa_canvas_config_set_optimizations (ChafaCanvasConfig *config, ChafaOptimizations optimizations)
+{
+    g_return_if_fail (config != NULL);
+    g_return_if_fail (config->refs > 0);
+
+    config->optimizations = optimizations;
+}
