@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2018-2020 Hans Petter Jansson
+/* Copyright (C) 2018-2021 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that turns images into character art.
  *
@@ -40,6 +40,12 @@ typedef struct
 }
 ChafaColor;
 
+typedef struct
+{
+    ChafaColor colors [2];
+}
+ChafaColorPair;
+
 typedef struct
 {
     union
@@ -68,6 +74,18 @@ chafa_color8_store_to_rgba8 (ChafaColor col, gpointer p)
     *p32 = cc.u.u32;
 }
 
+static inline ChafaColor
+chafa_color_average_2 (ChafaColor color_a, ChafaColor color_b)
+{
+    ChafaColor avg = { 0 };
+
+    CHAFA_COLOR8_U32 (avg) =
+        ((CHAFA_COLOR8_U32 (color_a) >> 1) & 0x7f7f7f7f)
+        + ((CHAFA_COLOR8_U32 (color_b) >> 1) & 0x7f7f7f7f);
+
+    return avg;
+}
+
 typedef struct
 {
     ChafaColor col [CHAFA_COLOR_SPACE_MAX];
