@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2018-2020 Hans Petter Jansson
+/* Copyright (C) 2018-2021 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that turns images into character art.
  *
@@ -76,6 +76,8 @@ typedef struct
     gint cell_width, cell_height;
     gdouble font_ratio;
     gint work_factor;
+    gint optimization_level;
+    ChafaOptimizations optimizations;
     guint32 fg_color;
     gboolean fg_color_set;
     guint32 bg_color;
@@ -89,11 +91,13 @@ typedef struct
     /* Applied after FPS is determined. If final value >= ANIM_FPS_MAX,
      * eliminate interframe delay altogether. */
     gdouble anim_speed_multiplier;
+
+    ChafaTermInfo *term_info;
 }
 GlobalOptions;
 
 static GlobalOptions options;
-static gboolean interrupted_by_user;
+static volatile sig_atomic_t interrupted_by_user = FALSE;
 
 static void
 sigint_handler (G_GNUC_UNUSED int sig)
@@ -186,7 +190,7 @@ collect_variable_arguments (int *argc, char **argv [], gint first_arg)
 }
 
 static const gchar copyright_notice [] =
-    "Copyright (C) 2018-2020 Hans Petter Jansson et al.\n"
+    "Copyright (C) 2018-2021 Hans Petter Jansson et al.\n"
     "Incl. libnsgif copyright (C) 2004 Richard Wilson, copyright (C) 2008 Sean Fox\n\n"
     "This is free software; see the source for copying conditions.  There is NO\n"
     "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n";
@@ -248,6 +252,10 @@ print_summary (void)
     "                     font file supported by FreeType (TTF, PCF, etc).\n"
     "      --invert       Invert video. For display with bright backgrounds in\n"
     "                     color modes 2 and none. Swaps --fg and --bg.\n"
+    "  -O, --optimize=NUM  Compress the output by using control sequences\n"
+    "                     intelligently [0-9]. 0 disables, 9 enables every\n"
+    "                     available optimization. Defaults to 5, except for when\n"
+    "                     used with \"-c none\", where it defaults to 0.\n"
     "  -p, --preprocess=BOOL  Image preprocessing [on, off]. Defaults to on with 16\n"
     "                     colors or lower, off otherwise.\n"
     "  -s, --size=WxH     Set maximum output dimensions in columns and rows. By\n"
@@ -574,6 +582,8 @@ parse_glyph_file_arg (G_GNUC_UNUSED const gchar *option_name, const gchar *value
     }
 
     font_loader = font_loader_new_from_mapping (file_mapping);
+    file_mapping = NULL;  /* Font loader owns it now */
+
     if (!font_loader)
     {
         g_set_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE,
@@ -581,9 +591,6 @@ parse_glyph_file_arg (G_GNUC_UNUSED const gchar *option_name, const gchar *value
         goto out;
     }
 
-    /* Font loader owns it now */
-    file_mapping = NULL;
-
     while (font_loader_get_next_glyph (font_loader, &c, &c_bitmap, &width, &height))
     {
         chafa_symbol_map_add_glyph (options.symbol_map, c,
@@ -770,106 +777,53 @@ tty_options_deinit (void)
     tcsetattr (STDIN_FILENO, TCSANOW, &saved_termios);
 }
 
-/* I really would've preferred to use termcap, but termcap contents
- * and the TERM env var are often unreliable/unrepresentative, so
- * instead we have this.
- *
- * If you're getting poor results, I'd love to hear about it so it can
- * be improved. */
 static void
-detect_terminal_modes (ChafaCanvasMode *mode_out, ChafaPixelMode *pixel_mode_out)
+detect_terminal (ChafaTermInfo **term_info_out, ChafaCanvasMode *mode_out, ChafaPixelMode *pixel_mode_out)
 {
-    const gchar *term;
-    const gchar *colorterm;
-    const gchar *vte_version;
-    const gchar *tmux;
-    ChafaCanvasMode mode = CHAFA_CANVAS_MODE_INDEXED_240;
-    ChafaPixelMode pixel_mode = CHAFA_PIXEL_MODE_SYMBOLS;
+    ChafaCanvasMode mode;
+    ChafaPixelMode pixel_mode;
+    ChafaTermInfo *term_info;
+    ChafaTermInfo *fallback_info;
+    gchar **envp;
 
-    term = g_getenv ("TERM");
-    if (!term) term = "";
+    envp = g_get_environ ();
+    term_info = chafa_term_db_detect (chafa_term_db_get_default (), envp);
 
-    colorterm = g_getenv ("COLORTERM");
-    if (!colorterm) colorterm = "";
-
-    vte_version = g_getenv ("VTE_VERSION");
-    if (!vte_version) vte_version = "";
-
-    tmux = g_getenv ("TMUX");
-    if (!tmux) tmux = "";
-
-    /* Some terminals set COLORTERM=truecolor. However, this env var can
-     * make its way into environments where truecolor is not desired
-     * (e.g. screen sessions), so check it early on and override it later. */
-    if (!g_ascii_strcasecmp (colorterm, "truecolor")
-        || !g_ascii_strcasecmp (colorterm, "gnome-terminal")
-        || !g_ascii_strcasecmp (colorterm, "xfce-terminal"))
+    if (chafa_term_info_have_seq (term_info, CHAFA_TERM_SEQ_SET_COLOR_FGBG_DIRECT)
+        && chafa_term_info_have_seq (term_info, CHAFA_TERM_SEQ_SET_COLOR_FG_DIRECT)
+        && chafa_term_info_have_seq (term_info, CHAFA_TERM_SEQ_SET_COLOR_BG_DIRECT))
         mode = CHAFA_CANVAS_MODE_TRUECOLOR;
-
-    /* In a modern VTE we can rely on VTE_VERSION. It's a great terminal emulator
-     * which supports truecolor. */
-    if (strlen (vte_version) > 0)
-        mode = CHAFA_CANVAS_MODE_TRUECOLOR;
-
-    /* Terminals that advertise 256 colors usually support truecolor too,
-     * (VTE, xterm) although some (xterm) may quantize to an indexed palette
-     * regardless. */
-    if (!strcmp (term, "xterm-256color")
-        || !strcmp (term, "xterm-kitty"))
-        mode = CHAFA_CANVAS_MODE_TRUECOLOR;
-
-    /* mlterm's truecolor support seems to be broken; it looks like a color
-     * allocation issue. This affects character cells, but not sixels.
-     *
-     * yaft supports sixels and truecolor escape codes, but it remaps cell
-     * colors to a 256-color palette. */
-    if (!strcmp (term, "mlterm")
-        || !strcmp (term, "yaft")
-        || !strcmp (term, "yaft-256color"))
-    {
-        /* The default canvas mode is truecolor for sixels. 240 colors is
-         * the default for symbols. */
+    else if (chafa_term_info_have_seq (term_info, CHAFA_TERM_SEQ_SET_COLOR_FGBG_256)
+        && chafa_term_info_have_seq (term_info, CHAFA_TERM_SEQ_SET_COLOR_FG_256)
+        && chafa_term_info_have_seq (term_info, CHAFA_TERM_SEQ_SET_COLOR_BG_256))
         mode = CHAFA_CANVAS_MODE_INDEXED_240;
+    else if (chafa_term_info_have_seq (term_info, CHAFA_TERM_SEQ_SET_COLOR_FGBG_16)
+        && chafa_term_info_have_seq (term_info, CHAFA_TERM_SEQ_SET_COLOR_FG_16)
+        && chafa_term_info_have_seq (term_info, CHAFA_TERM_SEQ_SET_COLOR_BG_16))
+        mode = CHAFA_CANVAS_MODE_INDEXED_16;
+    else if (chafa_term_info_have_seq (term_info, CHAFA_TERM_SEQ_INVERT_COLORS)
+             && chafa_term_info_have_seq (term_info, CHAFA_TERM_SEQ_RESET_ATTRIBUTES))
+        mode = CHAFA_CANVAS_MODE_FGBG_BGFG;
+    else
+        mode = CHAFA_CANVAS_MODE_FGBG;
+
+    if (chafa_term_info_have_seq (term_info, CHAFA_TERM_SEQ_BEGIN_SIXELS)
+        && chafa_term_info_have_seq (term_info, CHAFA_TERM_SEQ_END_SIXELS))
         pixel_mode = CHAFA_PIXEL_MODE_SIXELS;
-    }
+    else
+        pixel_mode = CHAFA_PIXEL_MODE_SYMBOLS;
 
-    /* rxvt 256-color really is 256 colors only */
-    if (!strcmp (term, "rxvt-unicode-256color"))
-        mode = CHAFA_CANVAS_MODE_INDEXED_240;
-
-    /* Regular rxvt supports 16 colors at most */
-    if (!strcmp (term, "rxvt-unicode"))
-        mode = CHAFA_CANVAS_MODE_INDEXED_16;
-
-    /* 'screen' does not like truecolor at all, but 256 colors works fine.
-     * Sometimes we'll see the outer terminal appended to the TERM string,
-     * like so: screen.xterm-256color */
-    if (!strncmp (term, "screen", 6))
-    {
-        mode = CHAFA_CANVAS_MODE_INDEXED_240;
-
-        /* 'tmux' also sets TERM=screen, but it supports truecolor codes.
-         * You may have to add the following to .tmux.conf to prevent
-         * remapping to 256 colors:
-         *
-         * tmux set-option -ga terminal-overrides ",screen-256color:Tc" */
-        if (strlen (tmux) > 0)
-            mode = CHAFA_CANVAS_MODE_TRUECOLOR;
-    }
-
-    /* If TERM is "linux", we're probably on the Linux console, which supports
-     * 16 colors only. It also sets COLORTERM=1.
-     *
-     * https://github.com/torvalds/linux/commit/cec5b2a97a11ade56a701e83044d0a2a984c67b4
-     *
-     * In theory we could emit truecolor codes and let the console remap,
-     * but we get better results if we do the conversion ourselves, since we
-     * can apply preprocessing and exotic color spaces. */
-    if (!strcmp (term, "linux"))
-        mode = CHAFA_CANVAS_MODE_INDEXED_16;
+    /* Make sure we have fallback sequences in case the user forces
+     * a mode that's technically unsupported by the terminal. */
+    fallback_info = chafa_term_db_get_fallback_info (chafa_term_db_get_default ());
+    chafa_term_info_supplement (term_info, fallback_info);
+    chafa_term_info_unref (fallback_info);
 
+    *term_info_out = term_info;
     *mode_out = mode;
     *pixel_mode_out = pixel_mode;
+
+    g_strfreev (envp);
 }
 
 static gboolean
@@ -900,6 +854,7 @@ parse_options (int *argc, char **argv [])
         { "font-ratio",  '\0', 0, G_OPTION_ARG_CALLBACK, parse_font_ratio_arg,  "Font ratio", NULL },
         { "glyph-file",  '\0', 0, G_OPTION_ARG_CALLBACK, parse_glyph_file_arg,  "Glyph file", NULL },
         { "invert",      '\0', 0, G_OPTION_ARG_NONE,     &options.invert,       "Invert foreground/background", NULL },
+        { "optimize",    'O',  0, G_OPTION_ARG_INT,      &options.optimization_level,  "Optimization", NULL },
         { "preprocess",  'p',  0, G_OPTION_ARG_CALLBACK, parse_preprocess_arg,  "Preprocessing", NULL },
         { "work",        'w',  0, G_OPTION_ARG_INT,      &options.work_factor,  "Work factor", NULL },
         { "size",        's',  0, G_OPTION_ARG_CALLBACK, parse_size_arg,        "Output size", NULL },
@@ -935,7 +890,7 @@ parse_options (int *argc, char **argv [])
     options.fill_symbol_map = chafa_symbol_map_new ();
 
     options.is_interactive = isatty (STDIN_FILENO) && isatty (STDOUT_FILENO);
-    detect_terminal_modes (&canvas_mode, &pixel_mode);
+    detect_terminal (&options.term_info, &canvas_mode, &pixel_mode);
     options.mode = CHAFA_CANVAS_MODE_MAX;  /* Unset */
     options.pixel_mode = pixel_mode;
     options.dither_mode = CHAFA_DITHER_MODE_NONE;
@@ -943,15 +898,16 @@ parse_options (int *argc, char **argv [])
     options.dither_grain_height = -1;  /* Unset */
     options.dither_intensity = 1.0;
     options.preprocess = TRUE;
-    options.color_extractor = CHAFA_COLOR_EXTRACTOR_MEDIAN;
+    options.color_extractor = CHAFA_COLOR_EXTRACTOR_AVERAGE;
     options.color_space = CHAFA_COLOR_SPACE_RGB;
     options.width = 80;
     options.height = 25;
     options.font_ratio = -1.0;  /* Unset */
     options.work_factor = 5;
+    options.optimization_level = G_MININT;  /* Unset */
     options.fg_color = 0xffffff;
     options.bg_color = 0x000000;
-    options.transparency_threshold = -1.0;
+    options.transparency_threshold = G_MAXDOUBLE;  /* Unset */
     options.file_duration_s = G_MAXDOUBLE;
     options.anim_fps = -1.0;
     options.anim_speed_multiplier = 1.0;
@@ -998,6 +954,15 @@ parse_options (int *argc, char **argv [])
         return FALSE;
     }
 
+    if (options.transparency_threshold == G_MAXDOUBLE)
+        options.transparency_threshold = 0.5;
+
+    if (options.transparency_threshold < 0.0 || options.transparency_threshold > 1.0)
+    {
+        g_printerr ("%s: Transparency threshold %.1lf is not in the range [0.0-1.0].\n", options.executable_name, options.transparency_threshold);
+        return FALSE;
+    }
+
     if (options.show_version)
     {
         print_version ();
@@ -1048,6 +1013,30 @@ parse_options (int *argc, char **argv [])
     if (options.mode != CHAFA_CANVAS_MODE_FGBG && !options.symbols_specified)
         chafa_symbol_map_remove_by_tags (options.symbol_map, CHAFA_SYMBOL_TAG_INVERTED);
 
+    /* If optimization level is unset, enable optimizations. However, we
+     * leave them off for FGBG mode, since control sequences may be
+     * unexpected in this mode unless explicitly asked for. */
+    if (options.optimization_level == G_MININT)
+        options.optimization_level = (options.mode == CHAFA_CANVAS_MODE_FGBG) ? 0 : 5;
+
+    if (options.optimization_level < 0 || options.optimization_level > 9)
+    {
+        g_printerr ("%s: Optimization level %d is not in the range [0-9].\n",
+                    options.executable_name, options.optimization_level);
+        return FALSE;
+    }
+
+    /* Translate optimization level to flags */
+
+    options.optimizations = CHAFA_OPTIMIZATION_NONE;
+
+    if (options.optimization_level >= 1)
+        options.optimizations |= CHAFA_OPTIMIZATION_REUSE_ATTRIBUTES;
+    if (options.optimization_level >= 6)
+        options.optimizations |= CHAFA_OPTIMIZATION_REPEAT_CELLS;
+    if (options.optimization_level >= 7)
+        options.optimizations |= CHAFA_OPTIMIZATION_SKIP_CELLS;
+
     g_option_context_free (context);
 
     return result;
@@ -1134,9 +1123,11 @@ build_string (ChafaPixelType pixel_type, const guint8 *pixels,
      * get the work factor. */
     chafa_canvas_config_set_work_factor (config, (options.work_factor - 1) / 8.0);
 
+    chafa_canvas_config_set_optimizations (config, options.optimizations);
+
     canvas = chafa_canvas_new (config);
     chafa_canvas_draw_all_pixels (canvas, pixel_type, pixels, src_width, src_height, src_rowstride);
-    gs = chafa_canvas_build_ansi (canvas);
+    gs = chafa_canvas_print (canvas, options.term_info);
 
     chafa_canvas_unref (canvas);
     chafa_canvas_config_unref (config);
@@ -1296,6 +1287,8 @@ run_magickwand (const gchar *filename, gboolean is_first_file, gboolean is_first
     gint loop_n = 0;
     FileMapping *file_mapping;
     XwdLoader *xwd_loader;
+    gchar buf [CHAFA_TERM_SEQ_LENGTH_MAX * 2 + 1];
+    gchar *p0;
 
     timer = g_timer_new ();
 
@@ -1447,27 +1440,30 @@ run_magickwand (const gchar *filename, gboolean is_first_file, gboolean is_first
                     g_free ((void *) pixels);
             }
 
+            p0 = buf;
+
             if (options.clear)
             {
                 if (is_first_frame)
                 {
                     /* Clear */
-                    printf ("\x1b[2J");
+                    p0 = chafa_term_info_emit_clear (options.term_info, p0);
                 }
 
                 /* Home cursor between frames */
-                printf ("\x1b[0f");
+                p0 = chafa_term_info_emit_cursor_to_top_left (options.term_info, p0);
             }
             else if (!is_first_frame)
             {
                 /* Cursor up N steps */
-                printf ("\x1b[%dA", frame->dest_height);
+                p0 = chafa_term_info_emit_cursor_up (options.term_info, p0, frame->dest_height);
             }
 
             /* Put a blank line between files in non-clear mode */
             if (is_first_frame && !options.clear && !is_first_file)
-                fputc ('\n', stdout);
+                *(p0++) = '\n';
 
+            fwrite (buf, sizeof (gchar), p0 - buf, stdout);
             fwrite (frame->gs->str, sizeof (gchar), frame->gs->len, stdout);
 
             /* No linefeed after frame in sixel mode */
@@ -1526,6 +1522,8 @@ run_gif (const gchar *filename, gboolean is_first_file, gboolean is_first_frame,
     Group group = { NULL };
     GList *l;
     gint loop_n = 0;
+    gchar buf [CHAFA_TERM_SEQ_LENGTH_MAX * 2 + 1];
+    gchar *p0;
 
     timer = g_timer_new ();
 
@@ -1594,27 +1592,30 @@ run_gif (const gchar *filename, gboolean is_first_file, gboolean is_first_frame,
                 gif_loader_next_frame (loader);
             }
 
+            p0 = buf;
+
             if (options.clear)
             {
                 if (is_first_frame)
                 {
                     /* Clear */
-                    printf ("\x1b[2J");
+                    p0 = chafa_term_info_emit_clear (options.term_info, p0);
                 }
 
                 /* Home cursor between frames */
-                printf ("\x1b[0f");
+                p0 = chafa_term_info_emit_cursor_to_top_left (options.term_info, p0);
             }
             else if (!is_first_frame)
             {
                 /* Cursor up N steps */
-                printf ("\x1b[%dA", frame->dest_height);
+                p0 = chafa_term_info_emit_cursor_up (options.term_info, p0, frame->dest_height);
             }
 
             /* Put a blank line between files in non-clear mode */
             if (is_first_frame && !options.clear && !is_first_file)
-                fputc ('\n', stdout);
+                *(p0++) = '\n';
 
+            fwrite (buf, sizeof (gchar), p0 - buf, stdout);
             fwrite (frame->gs->str, sizeof (gchar), frame->gs->len, stdout);
 
             /* No linefeed after frame in sixel mode */
@@ -1780,5 +1781,7 @@ main (int argc, char *argv [])
         chafa_symbol_map_unref (options.symbol_map);
     if (options.fill_symbol_map)
         chafa_symbol_map_unref (options.fill_symbol_map);
+    if (options.term_info)
+        chafa_term_info_unref (options.term_info);
     return ret;
 }
